/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.singleapp;

import java.io.Serializable;

/**
 * A singleton application that will not start if another instance is active.
 * Any application that wants this behavior must implement these methods.
 * If no inter-application messaging is needed then "send" should return null and "recv" should do nothing.
 *
 * @author  murali
 */
public interface Deferable extends Runnable {

    //------------------------------------------------------------
    //- Class Variables


    //------------------------------------------------------------
    //- Class Interface Functions

    /**
     * Callback to handle the situation where another instance was running
     * earlier, and this instance attempted to start but deferred to that.
     * This method can contain any logic needed to handle handoff data
     * between the two.
     *
     * @return  A serializable object that communicates the handoff from the dying application to the surviving one.
     */
    Serializable sendFromDeferred();

    /**
     * Callback to handle the situation where this instance is running, and
     * a second instance attempted to start but deferred to this.  This
     * method can contain any logic needed to handle handoff data between
     * the two.
     *
     * @param pObject  A serializable object that communicates the handoff from the dying application to the surviving one.
     */
    void recvFromDeferred(Object pObject);


}
