package com.notuvy.singleapp;

import org.apache.log4j.Logger;

import java.io.Serializable;
import java.util.Date;

/**
 * Basic description
 *
 * @author murali
 */
public class DummyApp {

    //------------------------------------------------------------
    //- Class Variables

    static {
        System.setProperty("log4j.configuration", "singleapp_log4j.xml");
    }

    protected static final Logger LOG = Logger.getLogger("com.notuvy.singleapp");

    private static final int PORT = 39234;


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    protected boolean alive = true;


    //------------------------------------------------------------
    //- Constructors

    private DummyApp() {
    }


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        LOG.info("Started: " + new Date());
        while (alive) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                // Ignore
            }
        }
        LOG.info("Ended: " + new Date());
    }


    //------------------------------------------------------------
    //- Inner Classes

    public static class Defer extends DummyApp implements Deferable {

        public Serializable sendFromDeferred() {
            return System.currentTimeMillis();
        }

        public void recvFromDeferred(Object pObject) {
            Long token = (Long) pObject;
            LOG.info(String.format("Other deferred [%d]: [%s]", token, new Date()));
        }

    }

    public static class Preemt extends DummyApp implements Preemptable {

        public void gracefulExitForPreempt() {
            alive = false;
        }

        public Serializable sendToPreemptor() {
            return null;
        }

        public void recvFromPreempted(Object pObject) {
        }
    }


    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            Deferable defer = null;
            Preemptable preempt = null;
            for (String arg : args) {
                if ("defer".equalsIgnoreCase(arg) || "d".equalsIgnoreCase(arg)) {
                    defer = new Defer();
                } else if ("preempt".equalsIgnoreCase(arg) || "p".equalsIgnoreCase(arg)) {
                    preempt = new Preemt();
                }
            }
            if ((defer == null) && (preempt == null)) {
                LOG.warn("Usage: singleapp-dummy <defer|preempt>");
            }

            if (defer != null) {
                SingletonApplicationLauncher.onPort(PORT).launchDefer(defer);
            } else if (preempt != null) {
                SingletonApplicationLauncher.onPort(PORT).launchPreempt(preempt);
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error", thr);
        }
    }


}
