package com.notuvy.singleapp;

import java.io.Serializable;

/**
 * An example of the Deferal pattern.
 *
 * @author murali
 */
public class DummyWebBrowser implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    static {
        System.setProperty("log4j.configuration", "singleapp_log4j.xml");
    }


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final String initialUrl;
    private boolean alive = true;


    //------------------------------------------------------------
    //- Constructors

    public DummyWebBrowser(String pInitialUrl) {
        initialUrl = pInitialUrl;
    }


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public void openURL(String pUrl) {
        System.out.println("URL: " + pUrl);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        openURL(initialUrl);

        while (alive) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                // nothing
            }
        }
    }


    //------------------------------------------------------------
    //- Inner Classes

    static class Launcher implements Deferable {
        private final DummyWebBrowser browser;

        Launcher(DummyWebBrowser pBrowser) {
            browser = pBrowser;
        }

        public void run() {
            browser.run();
        }

        public Serializable sendFromDeferred() {
            return browser.initialUrl;
        }

        public void recvFromDeferred(Object pObject) {
            String url = (String) pObject;
            browser.openURL(url);
        }
    }


    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        DummyWebBrowser browser = new DummyWebBrowser(args[0]);
        SingletonApplicationLauncher.onPort(10431).launchDefer(new Launcher(browser));
    }

}
