package com.notuvy.singleapp;

import com.notuvy.singleapp.multiuser.MultiUserDefer;
import com.notuvy.singleapp.multiuser.MultiUserPreempt;
import org.apache.log4j.Logger;

/**
 * Basic description
 *
 * @author murali
 */
public class SingletonApplicationLauncher {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.singleapp");


    //------------------------------------------------------------
    //- Class Functions

    public static SingletonApplicationLauncher onPort(int pPort) {
        return new SingletonApplicationLauncher(pPort);
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final int fPort;
    private Strategy fStrategy = Strategy.MULTI_USER;
    private boolean fAllowSystemExit = true;
    Enforcer fEnforcer;


    //------------------------------------------------------------
    //- Constructors

    private SingletonApplicationLauncher(int pPort) {
        fPort = pPort;
    }


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors

    // This is here for unit tests.  In real life, we need a way to force the exit to guarantee preemption.
    protected SingletonApplicationLauncher setAllowSystemExit(boolean pAllowSystemExit) {
        fAllowSystemExit = pAllowSystemExit;

        return this;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public SingletonApplicationLauncher setSingleUser() {
        fStrategy = Strategy.SINGLE_USER;

        return this;
    }

    public SingletonApplicationLauncher setMultiUser() {
        fStrategy = Strategy.MULTI_USER;

        return this;
    }

    public void launchDefer(Deferable pDeferable) {
        if (fStrategy.equals(Strategy.MULTI_USER)) {
            fEnforcer = new MultiUserDefer(fPort, pDeferable);
        } else {
            fEnforcer = new SingleUserEnforcer.Defer(fPort, pDeferable);
        }
        fEnforcer.attempt();
    }

    public void launchPreempt(Preemptable pPreemptable) {
        if (fStrategy.equals(Strategy.MULTI_USER)) {
            fEnforcer = new MultiUserPreempt(fPort, pPreemptable, fAllowSystemExit);
        } else {
            fEnforcer = new SingleUserEnforcer.Preempt(fPort, pPreemptable, fAllowSystemExit);
        }
        fEnforcer.attempt();
    }

    // This is here for unit tests.  Usually for DEFER, the listener dies when the process dies.
    // However, this is not relevant for unit tests because the process will not die.  So we need
    // a mechanism to shutdown the listener and its thread.
    protected void shutdown() {
        if (fEnforcer != null) {
            fEnforcer.shutdownServer();
            fEnforcer = null;
        }
    }


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes

    public enum Strategy {
        SINGLE_USER,
        MULTI_USER
    }


    //------------------------------------------------------------
    //- Main


}
